#!/usr/bin/python

import ctypes
import os
import threading
import time
import sys, tty, termios

import array as arr
import asyncio

scanChanCnt = 3

BOOLEAN = ctypes.c_ubyte
FALSE = 0
TRUE = 1
# single chan regular token
CMND_SINGLE_CHAN_SCAN = 0x89 
# single chan with digin token
CMND_SINGLE_CHAN_DIGIN_SCAN = 0x8D  
# multi chan regular token
CMND_MULTI_CHAN_SCAN = 0x8B 
# multi chan with cal token
CMND_MULTI_CHAN_CAL_SCAN = 0x8C 
# multi chan with digin token	
CMND_MULTI_CHAN_DIGIN_SCAN = 0x8E 
# multi chan with cal and digin token
CMND_MULTI_CHAN_CAL_DIGIN_SCAN = 0x8F 


def funcDoScan():
  global LL_USB_Lib
  global scanType
  global scanChanCnt
	

  class scanStruct(ctypes.Structure):
     _fields_ = [('fGotBuffWrap', ctypes.c_ubyte), ('inThread', ctypes.c_int),
       ('isScanning', ctypes.c_int), ('fScanThreadLoopDone', ctypes.c_ubyte),
       ('scanBuff', ctypes.POINTER(ctypes.c_int)),
       ('diginBuff', ctypes.POINTER(ctypes.c_ubyte)),
       ('uiStackedScans', ctypes.c_uint), ('uiHeadPntr', ctypes.c_uint),
       ('uiTailPntr', ctypes.c_uint), ('uiBufferSizeMin', ctypes.c_uint),
       ('uiBufferSizeMax', ctypes.c_uint), ('uiScanToken', ctypes.c_uint),
       ('uiScanChans', ctypes.c_uint), ('uiPointsInBuff', ctypes.c_uint)]


  scanStruct = scanStruct.in_dll(LL_USB_Lib, 'scanStruct')

  scanStruct.inThread = 1
  scanStruct.uiBufferSizeMin = 500; scanStruct.uiBufferSizeMax = 500
  # for testing scanStruct.uiBufferSizeMin = 15; scanStruct.uiBufferSizeMax = 15
	
  scanStruct.uiHeadPntr = 0; scanStruct.uiTailPntr = 0
  scanStruct.isScanning = FALSE
  scanStruct.fGotBuffWrap = FALSE 
  scanStruct.fScanThreadLoopDone = FALSE  
  scanStruct.uiScanToken = CMND_SINGLE_CHAN_SCAN
 
  #LOOK HERE: tested with various scan types, the final one here being CMND_MULTI_CHAN_CAL_DIGIN_SCAN
  # note that scanStruct.uiScanChans also needs adjusted for scan type and that the "calibration"
  # channel if doing a "cal-scan" is not counted here in channel count!
  
  #LOOK HERE DIGIN
  #NOTE below is how the bytes are returned in both the local and library (via firmware) for each scan type
  #local buffer looks like: [chan data][chan data][chan data][chan data]
  #library buffer looks like (local array is 3-byte integer):
  #[chan1 byte1][chan1 byte2][chan1 byte3][chan1 byte1][chan1 byte2][chan1 byte3]
#  scanStruct.uiScanToken = CMND_SINGLE_CHAN_SCAN
  #local #buffer looks like: [chan 1][chan 2][chan 3][chan . . .]
  #library buffer looks like:
  #[chan1 byte1][chan1 byte2][chan1 byte3][chan2 byte1][chan2 byte2][chan2 byte3]
#  scanStruct.uiScanToken = CMND_MULTI_CHAN_SCAN
  #local #buffer looks like: [chan 1][offset cal][chan 2][offset cal][chan . . .]
  #library buffer looks like:
  #[chan1 byte1][chan1 byte2][chan1 byte3][chan2 byte1][chan2 byte2][chan2 byte3][offset byte1][offset byte2][offset byte3]
#  scanStruct.uiScanToken = CMND_MULTI_CHAN_CAL_SCAN
  #local scan buffer looks like: [chan data][chan data][chan data][chan data]
  #local digin buffer looks like: [chan digin][chan digin][chan digin][chan digin]
  #library buffer looks like:
  #[chan byte1][chan byte2][chan byte3][digital input byte][chan byte1][chan byte2][chan byte3][digital input byte]
#  scanStruct.uiScanToken = CMND_SINGLE_CHAN_DIGIN_SCAN
  #local scan buffer looks like: [chan1 data][chan2 data][offset data]
  #local digin buffer looks like: [chan2 digin][chan2 digin]
  #library buffer looks like:
  #[chan1 byte1][chan1 byte2][chan1 byte3][digital input byte]
  #  [chan2 byte1][chan2 byte2][chan2 byte3][digital input byte][offset byte1][offset byte2][offset byte3]
  #  [digital input byte]
  #scanStruct.uiScanToken = CMND_MULTI_CHAN_CAL_DIGIN_SCAN
  #7-23-24
  #scanStruct.uiScanToken = CMND_MULTI_CHAN_DIGIN_SCAN
  
  #LOOK HERE added from command line input 7-26-24 but not completely functional yet
  scanStruct.uiScanToken = scanType
  
  #LOOK HERE added from command line input 7-26-24 but not completely functional yet
  scanStruct.uiScanChans = scanChanCnt
  
  scanStruct.uiStackedScans = 1 #MUST ALWAYS BE AT LEAST 1
  ## scan struct variable init end  ===

  #following just defaults, will be asked for both when the scan is started
  #search "doSetNumberOfScans" for explanation using each of the following
  fContinuousScan = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fContinuousScan')
  fContinuousScan.value = TRUE	#this trumps the others. If true, uiScansToDo is ignored	
  fTimedScan = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fTimedScan')
  fTimedScan.value = FALSE
  uiScansToDo = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScansToDo')
  uiScansToDo.value = 100		#"scans" not data points (default to 100)
  fDoDispData = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fDoDispData')
  fDoDispData.value = TRUE
  fContScanDone = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fContScanDone')
  fContScanDone.value = FALSE 
	
  fEndScanTokenSent = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fEndScanTokenSent')
  fEndScanTokenSent.value = FALSE 
  fScanThreadActive = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanThreadActive')
  fScanThreadActive.value = FALSE 

  uiScanHours = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScanHours')
  uiScanHours.value = 1		#just any default
  fScanHours = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanHours')
  fScanHours.value = FALSE		#just any default 
  uiScanMinutes = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScanMinutes')
  uiScanMinutes.value = 1		#just any default 
  fScanMinutes = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanMinutes')
  fScanMinutes.value = FALSE	#just any default 
  uiScanSeconds = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScanSeconds')
  uiScanSeconds.value = 60		#just any default 
  fScanSeconds = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanSeconds')
  fScanSeconds.value = FALSE	#just any default 

  #LOOK HERE - default is 1
  fScanPointCnt = 1
  bAddForCalChan = 0

  fIsDigScan = ctypes.c_ubyte(0); fIsDigScan.value = FALSE
  if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN) 
    or (scanStruct.uiScanToken == CMND_MULTI_CHAN_DIGIN_SCAN)
    or (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN)):
      fIsDigScan.value = TRUE

  bAddForOffsetChan = ctypes.c_ubyte(0); bAddForOffsetChan.value = 0
  if((scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_SCAN)
    or (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN)):
    bAddForOffsetChan = 1;
    bAddForCalChan = 1;	

  bIsSingleChan = ctypes.c_ubyte(0); bIsSingleChan = 0
  if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_SCAN) 
      or (scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN)):
    bIsSingleChan = 1
	
  scanStruct.uiBufferSizeMax = (scanStruct.uiBufferSizeMin * scanStruct.uiStackedScans * 4)
  print ("scanStruct.uiBufferSizeMax: %d\n" % scanStruct.uiBufferSizeMax)
  scanStruct.scanBuff = (ctypes.c_int * (scanStruct.uiBufferSizeMax))() #allocate buffer
  for x in range(0, (scanStruct.uiBufferSizeMax)):
    scanStruct.scanBuff[x] = 2

  fScanPointCnt = scanStruct.uiScanChans + bAddForCalChan; 
   
  #LOOK HERE: doesn't hurt to just have this always allocated except extra memory usage
  # but can use, "if fIsDigScan.value == TRUE"
  scanStruct.diginBuff = (ctypes.c_ubyte * scanStruct.uiBufferSizeMax)() 


  ##################################################
  ## start of thread ###############################
  LL_USB_Lib.doScan.argtypes = [ctypes.c_void_p]
  LL_USB_Lib.doScan.restypes = ctypes.c_void_p
  testVal = ctypes.c_void_p(6) #made 6 but can be any val since it's not used

  # Create a new thread  
  scanThread = threading.Thread(target=LL_USB_Lib.doScan, args=(testVal,))
  # Using the daemon property set the thread as daemon before starting the thread so system won't lock if there is a scan problem
  scanThread.daemon = True
  # start the thread
  scanThread.start()

  uiScansToGet = 0
 
  if(scanStruct.uiScanToken == CMND_SINGLE_CHAN_SCAN): print("getting data for CMND_SINGLE_CHAN_SCAN\n")
  if(scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN): print("getting data for CMND_SINGLE_CHAN_DIGIN_SCAN\n")
  if(scanStruct.uiScanToken == CMND_MULTI_CHAN_SCAN): print("getting data for CMND_MULTI_CHAN_SCAN\n")
  if(scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_SCAN): print("getting data for CMND_MULTI_CHAN_CAL_SCAN\n")
  if(scanStruct.uiScanToken == CMND_MULTI_CHAN_DIGIN_SCAN): print("getting data for CMND_MULTI_CHAN_DIGIN_SCAN\n")
  if(scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN): print("getting data for CMND_MULTI_CHAN_CAL_DIGIN_SCAN\n")
 
  bLoopCnt = 0

  #LOOK HERE
	#make short for simple test, long for USB or power disconnect
	# so there is enough time to disconnect one or the other or both.
	# For the latter, also recommend slow data rate and multi-chan-digin 
	# scan to increase time of scan so it doesn't end before chance to
	# disconnect a cable
  #while(bLoopCnt < 2000):
  while(bLoopCnt < 10):
    if(fContScanDone.value == TRUE):
      break
			
    if((scanStruct.uiPointsInBuff) >= fScanPointCnt):
      print ("scan: %u  volts: " % bLoopCnt, end = " ")
      digTailSave = 0
      for x in range(fScanPointCnt):

        if (float((scanStruct.scanBuff[(scanStruct.uiTailPntr)] - 8388608) * 0.00000059604645) < 0):
          print ("%f " % float((scanStruct.scanBuff[(scanStruct.uiTailPntr)] - 8388608) * 0.00000059604645), end = " ")
        else:
          print (" %f " % float((scanStruct.scanBuff[(scanStruct.uiTailPntr)] - 8388608) * 0.00000059604645), end = " ")
					
				
        digTailSave = scanStruct.uiTailPntr	
				
        scanStruct.uiTailPntr += 1
        if(scanStruct.uiTailPntr == scanStruct.uiBufferSizeMax):
          scanStruct.uiTailPntr = 0

      if(fIsDigScan):
        print ("digin: %u " % (scanStruct.diginBuff[digTailSave]), end = " ")
      
      scanStruct.uiPointsInBuff -= (fScanPointCnt)
			
      print (" ")
      bLoopCnt+=1
			
			
  print (" ")  
  fContScanDone.value = TRUE 
 
  return			 



def main():
  global LL_USB_Lib
  global scanType
  global scanChanCnt
  
  LL_USB_Lib = ctypes.cdll.LoadLibrary('./libLL_USB30x.so')
 
  os.system("clear")
  devID = ctypes.c_ushort
	  
  #LOOK HERE: this is the M302 device ID
  print ("\n--> enter device ID: ", end = " ")
  devID = int(input())
  print ("")
  myDev = devID
  usDevID = devID
  #LOOK HERE: hard-coded for now
  rate = float(1000.0)
	
  print ("--> enter single channel chan for get-conversion below: ")
  print ("     suggest connecting to known voltage source to chosen channel for test further down")
  print ("valid: 1,2,3,4,5,6,7,8: ", end = " ")
	
  mainChan = int(input())
  if(mainChan > 8):
    print("**! invalid number entered for chan main channel - please try again later\n") 
    print("**! setting main channel to 8 for this test\n") 
  if(mainChan < 1):
    mainChan = 1
    print("**! 0 is not valid channel - setting channel to 1\n") 
    mainChan -= 1

  print ("--> make connection now if you want, then <enter> to continue")
  miscInput = input()

  mainChan -= 1
  bSingleChanScanChan = mainChan
  
  print ("--> enter scan type by number: ")
  print ("CMND_SINGLE_CHAN_SCAN = 1")
  print ("CMND_SINGLE_CHAN_DIGIN_SCAN = 2")
  print ("CMND_MULTI_CHAN_SCAN = 3")
  print ("CMND_MULTI_CHAN_CAL_SCAN = 4")
  print ("CMND_MULTI_CHAN_DIGIN_SCAN = 5")
  print ("CMND_MULTI_CHAN_DIGIN_CAL_SCAN = 6")
  print ("enter number here: ", end = " ")
  scanTypeTemp = input()
  print ("")
 
  isMultiChan = TRUE
	
  if(scanTypeTemp == '1'): 
    scanType = 0x89
    isMultiChan = FALSE
    print ("got CMND_SINGLE_CHAN_SCAN")
  elif(scanTypeTemp == '2'): 
    scanType = 0x8D
    isMultiChan = FALSE
    print ("got CMND_SINGLE_CHAN_DIGIN_SCAN")
  elif(scanTypeTemp == '3'): 
    scanType = 0x8B
    print ("got CMND_MULTI_CHAN_SCAN")
  elif(scanTypeTemp == '4'): 
    scanType = 0x8C
    print ("got CMND_MULTI_CHAN_CAL_SCAN")
  elif(scanTypeTemp == '5'): 
    scanType = 0x8E
    print ("got CMND_MULTI_CHAN_DIGIN_SCAN")
  elif(scanTypeTemp == '6'): 
    scanType = 0x8F
    print ("got CMND_MULTI_CHAN_DIGIN_CAL_SCAN")
  else: 
    print("**! invalid number entered for scan type - please start again\n")
    print("**! setting schan type to MULTI_CHAN_DIGIN_CAL_SCAN for you\n")
    scanType = 0x8F
	
  if isMultiChan == TRUE:	
    print ("--> enter number of scan channels for multi-chan scan below:")
    print ("--> enter 1-8: ", end = " ")
    scanChanCnt = int(input())
    if(scanChanCnt > 8):
      print("**! invalid number entered for chan count - please start again\n")  
      print("**! setting chan count to 3 for you\n")
      scanChanCnt = 3
		
    if((scanChanCnt > 7) & ((scanType == 0x8E) | (scanType == 0x8F))):
      scanChanCnt = 7 
	  
  else:
    scanChanCnt = 1
    
  rate = 1000
  dblActualRate = ctypes.c_double(rate)
	
  print ("\n--> rate set in code to %0.3f" % rate)
  LL_USB_Lib.EX_GetCalculatedRate.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetCalculatedRate.restypes = ctypes.c_ubyte
  LL_USB_Lib.EX_GetCalculatedRate(usDevID, dblActualRate)
  print ("--->  which has \"actual value\" of %f\n" % dblActualRate.value)

  print("--> hit <enter> to continue to sign-on", end = " ")
  miscInput = input()
  
  os.system("clear")
	
  print ("")

  LL_USB_Lib.EX_ConnectOneDevice.argtypes = [ctypes.c_ushort, ctypes.c_double, ctypes.c_ubyte]
  LL_USB_Lib.EX_ConnectOneDevice.restypes = ctypes.c_byte
  print ("\nsigning on (may take a while) - WAIT . . .")
  if LL_USB_Lib.EX_ConnectOneDevice(usDevID, rate, 3):
    print ("  signed on\n")
  else:
    print ("not signed on\n")
		
    print ("will attempt reset - any key to continue")
    miscInput = input()
    print ("  resetting")
    LL_USB_Lib.resetDevFirmware.argtypes = [ctypes.c_ushort]
    LL_USB_Lib.resetDevFirmware.restypes = ctypes.c_ubyte
    if LL_USB_Lib.resetDevFirmware(myDev):
      print ("  reset success\n")
    else:
      print ("reset failed - any key to exit")
      miscInput = input()
      return
    if LL_USB_Lib.EX_ConnectOneDevice(usDevID, rate, 3):
      print ("  signed on\n")
    else:
      print ("not signed on\n")
      return
			

  print ("*** digital outputs connected to digital input for following test")
  print ("--> make connection now if you want, then <enter> to continue")
  miscInput = input()

  #LOOK HERE - set this and everything else below according to user-input, or whatever
  # - just hard-coded here for testing 
  bDigout = ctypes.c_ubyte(255)
  print ("sending digout 255 - WAIT . . .")
  LL_USB_Lib.EX_SendDigout.argtypes = [ctypes.c_ushort, ctypes.c_ubyte]
  LL_USB_Lib.EX_SendDigout.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendDigout(usDevID, bDigout):	
    print ("  EX_SendDigout\n")
  else:
    print ("  failed sending digout\n")
    return
  
  bChan = ctypes.c_ubyte(mainChan)
  bLastDigin = ctypes.c_ubyte(0)
  print ("sending channel %d - WAIT . . ." % (mainChan + 1))
  LL_USB_Lib.EX_SendChan.argtypes = [ctypes.c_ushort, ctypes.c_ubyte, ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.EX_SendChan.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendChan(usDevID, bChan, bLastDigin):	
    print ("  sendChan success, lastDigin: %d\n" % bLastDigin.value)
  else:
    print ("  failed sending channel\n")
    return

  dblMilliVolts = ctypes.c_double(100)
  print ("getting a conversion for channel %d - WAIT . . ." % (mainChan + 1))
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts):	
    print ("  got volts chan0: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    return
	
  bChan = ctypes.c_ubyte(6)
  bLastDigin = ctypes.c_ubyte(0)
  print ("changing to  full-scale channel 6 - WAIT . . .")
  LL_USB_Lib.EX_SendChan.argtypes = [ctypes.c_ushort, ctypes.c_ubyte, ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.EX_SendChan.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendChan(usDevID, bChan, bLastDigin):	
    print ("  sendChan success, lastDigin: %d\n" % bLastDigin.value)
  else:
    print ("  failed sending channel\n")
    return
	
  print ("getting a conversion - WAIT . . .")
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts)	
  if LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts):	
    print ("  got volts chan6: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    return

  print ("** DAC3 is connected to chan 3 input for following test . . .")
  print ("--> make connection now if you want, then <enter> to continue")
  miscInput = input()

  bDAC = ctypes.c_ubyte(2)
  dblDACVolts = ctypes.c_double(3.5)
  print ("setting DAC3 to 3.5 volts - WAIT . . .")
  LL_USB_Lib.EX_SendDAC.argtypes = [ctypes.c_ushort, ctypes.c_double, ctypes.c_ubyte]
  LL_USB_Lib.EX_SendDAC.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendDAC(usDevID, dblDACVolts, bDAC):	
    print ("  sendDAC success")
  else:
    print ("  failed setting DAC\n")
    return  

  #LOOK HERE "some" delay needed for DAC to settle - wait is no longer in library
  # since it could interfere with scan
  time.sleep(.7)

  bChan = 2
  print ("changing to channel 3 (should be DAC3 volts)")
  LL_USB_Lib.EX_SendChan.argtypes = [ctypes.c_ushort, ctypes.c_ubyte, ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.EX_SendChan.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendChan(usDevID, bChan, bLastDigin):	
    print ("  sendChan success, lastDigin: %d\n" % bLastDigin.value)
  else:
    print ("  failed sending channel\n")
    return

  print ("getting a conversion - WAIT . . .")
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts):	
    print ("  got volts chan3: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    return
		
  print ("checking 'actual rate' when 1000Hz is requested - WAIT . . .")
  LL_USB_Lib.EX_GetCalculatedRate.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetCalculatedRate.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetCalculatedRate(usDevID, dblActualRate):	
    print ("  1000Hz would be: %f\n" % dblActualRate.value)
  else:
    print ("  failed getting actual rate for 1000Hz\n")
    return
  LL_USB_Lib.AllowPrintf(1) #allows printf of some notifications from within library

  #scan may end if USB cable disconnected, power loss, or other reason.
	#if this flag set, do not check end-scan codes when scan ends, otherwise, do
  fBadScanEnd = ctypes.c_byte.in_dll(LL_USB_Lib, 'fBadScanEnd')
  fBadScanEnd.value = FALSE	

  #this array will contain scan codes at end of scan - look at them if you want
	# - should be: 0, 23, 3, 2, 1
  paEndScanCodes = ctypes.POINTER(ctypes.c_ubyte)
  paEndScanCodes = paEndScanCodes.in_dll(LL_USB_Lib, 'paEndScanCodes')

	#this flag will be set to true if there was any scan error error in which the scan codes were not
	#returned. If this flag is not set, then wait for 
  fPossibleScanUSB_Err = ctypes.c_byte
  fPossibleScanUSB_Err = fPossibleScanUSB_Err.in_dll(LL_USB_Lib, 'fPossibleScanUSB_Err')

  #this flag will be set TRUE if/after library scan thread got correct end-scan codes. 
  fGotCorrectEndScanCodes = ctypes.c_byte
  fGotCorrectEndScanCodes = fGotCorrectEndScanCodes.in_dll(LL_USB_Lib, 'fGotCorrectEndScanCodes')

  #this flag will be set TRUE after scan thread is finised checking scan codes after scan end. 
	#best to check this flag if fPossibleScanUSB_Err is not set, before continuing to do anything
	#after scan has ended.
  #fScanThreadWait = ctypes.c_byte.in_dll(ctypes.c_ubyte)
  fScanThreadWait = ctypes.c_byte
  fScanThreadWait = fScanThreadWait.in_dll(LL_USB_Lib, 'fScanThreadWait')

  bDAC = ctypes.c_ubyte(2)

  print ("starting scan")
	
  funcDoScan()
	
  print ("--> done scan - checking error flag")
  #miscInput = input()
	
  if(fPossibleScanUSB_Err.value == FALSE):
    print ("no scan error - waiting for library to read end-scan codes\n")
    while(fScanThreadWait.value == TRUE):
      time.sleep(.7)
      #print (". ", end = " ")
  else:
    print ("possible scan error\n")
		  
  print ("\n")
	
  print ("from python app -> scan complete - returned from library thread ...")
  print ("--> from python app -> hit <enter> to continue testing")
  miscInput = input()
	
  bIsBrdConnected = ctypes.c_ubyte(0)
  bIsDevConnected = ctypes.c_ubyte(0)
  LL_USB_Lib.checkConnectLLabs.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_ubyte), ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.checkConnectLLabs.restypes = ctypes.c_byte
  if LL_USB_Lib.checkConnectLLabs(myDev, bIsBrdConnected, bIsDevConnected):
    print ("  checkConnectLLabs success")
    print ("  bIsBrdConnected: %d" % bIsBrdConnected.value)
    print ("  bIsDevConnected: %d" % bIsDevConnected.value)
  else:
    print ("  checkConnectLLabs failed\n")
		
		
  LL_USB_Lib.EX_ConnectOneDevice.restypes = ctypes.c_byte
  print ("\nfrom python app -> signing on (may take a while) - WAIT . . .")
  if LL_USB_Lib.EX_ConnectOneDevice(myDev, 100.0, 3):
    print ("  from python app -> signed on\n")
  else:
    print ("  from python app -> not signed on - will try again\n")
		
    #8-27-24 required for now after failed signon
    LL_USB_Lib.clearDevHandle.argtypes = [ctypes.c_ushort]
    LL_USB_Lib.clearDevHandle.restypes = ctypes.c_ubyte
    LL_USB_Lib.clearDevHandle(myDev)
	
    print ("from python app -> trying signon again\n")
    LL_USB_Lib.EX_ConnectOneDevice.restypes = ctypes.c_byte
    print ("\nsigning on (may take a while) - WAIT . . .")
    if LL_USB_Lib.EX_ConnectOneDevice(myDev, 100.0, 3):
      print ("  from python app -> signed on\n")
    else:
      print ("  from python app -> not signed on\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return
			
			
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	#below is to check system after a possible scan error to check system after power disconnect
  #+++++++++++++++++++++++++++++++++++++++==++++++++++++++++++++++++++++++++++++++++++++++++++			
  print ("changing to channel 3 which I've connected to DAC3")
  LL_USB_Lib.EX_SendChan.argtypes = [ctypes.c_ushort, ctypes.c_ubyte, ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.EX_SendChan.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendChan(myDev, bChan, bLastDigin):	
    print ("  sendChan success, lastDigin: %d\n" % bLastDigin.value)
  else:
    print ("  failed sending channel\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return

	#calling this here just to make it easier to comment the entire block when done testing
  bDAC = ctypes.c_ubyte(2)
  dblDACVolts = ctypes.c_double(1.0)
  print ("setting DAC3 to 1.0 volts - WAIT . . .")
  LL_USB_Lib.EX_SendDAC.argtypes = [ctypes.c_ushort, ctypes.c_double, ctypes.c_ubyte]
  LL_USB_Lib.EX_SendDAC.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendDAC(myDev, dblDACVolts, bDAC):	
    print ("  sendDAC success")
  else:
    print ("  failed setting DAC\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return

  time.sleep(.7)		

  print ("getting a conversion - WAIT . . .")
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(myDev, dblMilliVolts):	
    print ("  got volts chan3: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return


  dblDACVolts = ctypes.c_double(5.0)
  print ("setting DAC3 to 5.0 volts - WAIT . . .")
  LL_USB_Lib.EX_SendDAC.argtypes = [ctypes.c_ushort, ctypes.c_double, ctypes.c_ubyte]
  LL_USB_Lib.EX_SendDAC.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendDAC(myDev, dblDACVolts, bDAC):	
    print ("  sendDAC success")
  else:
    print ("  failed setting DAC\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return

  time.sleep(.7)		

  print ("getting a conversion - WAIT . . .")
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(myDev, dblMilliVolts):	
    print ("  got volts chan3: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return

  print ("--> from python app -> hit <enter> to continue test")
  miscInput = input()
	
	
  rate = 1000.0
  dblActualRate = ctypes.c_double(rate)
  print ("\n--> rate set in code to %0.3f" % rate)
  LL_USB_Lib.EX_GetCalculatedRate.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetCalculatedRate.restypes = ctypes.c_ubyte
  LL_USB_Lib.EX_GetCalculatedRate(usDevID, dblActualRate)
  print ("--->  which has \"actual value\" of %f\n" % dblActualRate.value)

  print("--> hit <enter> to continue to sign-on", end = " ")
  miscInput = input()
		
  print ("")

  LL_USB_Lib.EX_ConnectOneDevice.restypes = ctypes.c_byte
  print ("\nsigning on (may take a while) - WAIT . . .")
  if LL_USB_Lib.EX_ConnectOneDevice(usDevID, rate, 3):
    print ("  signed on\n")
  else:
    print ("not signed on\n")
		
    print ("will attempt reset - any key to continue")
    miscInput = input()
    print ("  resetting")
    LL_USB_Lib.resetDevFirmware.argtypes = [ctypes.c_ushort]
    LL_USB_Lib.resetDevFirmware.restypes = ctypes.c_ubyte
    if LL_USB_Lib.resetDevFirmware(myDev):
      print ("  reset success")
    else:
      print ("reset failed - any key to exit")
      miscInput = input()
      return
			
    print ("\nattempting signon again after reset\n")
    LL_USB_Lib.EX_ConnectOneDevice.restypes = ctypes.c_byte
    print ("\nsigning on (may take a while) - WAIT . . .")
    if LL_USB_Lib.EX_ConnectOneDevice(usDevID, rate, 3):
      print ("  signed on\n")
    else:
      print ("not signed on\n")
      return
	
  print ("getting a conversion - WAIT . . .")
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(myDev, dblMilliVolts):	
    print ("  got volts chan3: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    print ("--> from python app -> hit <enter> to exit app")
    miscInput = input()
    return
		
  print ("starting scan")
  funcDoScan()
	
  print ("done second scan - checking error flag\n")
	
  if(fPossibleScanUSB_Err.value == FALSE):
    print ("no scan error - waiting for library to read end-scan codes\n")
    while(fScanThreadWait.value == TRUE):
      time.sleep(.7)
  else:
    print ("possible scan error\n")
		  
  print ("\n")
	
  print ("library finished scan thread\n")
	
  print("--> hit <enter> to continue to exit app", end = " ")
  miscInput = input()

  return


if __name__ == '__main__':
  main()
